Lightweight Java Logger
=======================
Simple, lightweight, asynchronous java logging framework
--------------------------------------------------------

Do you ever been confused by Log4j's loggers hierarchy, messages patterns or complexity of configuration file? I do! And because of this, I wrote this framework.  
The main feature of Lightweight Java Logger (LJL) is simpleness. By default (without any configuration), LJL wrote log messages in console in async mode using the following pattern: `[$level$] $date$ $thread$ - $class$: $msg$` (`$date$` will use standart `java.util.Date.toString()` method).  
As you can see above, pattern has human readable format (instead of the mad Log4j's pattern format: `%d{ABSOLUTE} %5p %c{1}:%L` ⊙▂⊙).  
Core difference between Log4j and LJL is loggers hierarchy. In Log4j you define logger settings and all children loggers will inherit them (`com.foo` is the parent logger for `com.foo.bar`). In LJL you define independent loggers and configure them to write messages from package or class.  
LJL can dynamically load properties updates.

Example
-------
```java
Logger logger = LoggerFactory.getLogger(Clazz.class); 
logger.debug("Execute {} with {}", "example", "example argument");
```

Usages
------
LJL was written for the small and medium sized projects. If you need time-tested logging subsytem with the best performance, use Log4j. Otherwise, you have to try LJL ;D.  

Performance
-----------
For load testing I used [jLoad](https://github.com/dreambrother/jload).  
I used pattern `[$level$] $thread$ $class$ $date:yyyy-MM-dd HH:mm:ss$ $msg$` and file appender.
On my laptop (Intel Core2Duo P8400 @ 2.26GHz × 2, 3GB DDR2) one call took above 0.08 millisecond (Log4j took above 0.02 millisecond).

Configuration
-------------
As I said earlier, LJL has default configuration. If you want to customize configuration, the simpliest way to do it is to create file `ljl-properties.json`. For example, you want to log WARN (and above) messages from package `com.foo.bar` and all subpackages to file named `/logs/component.log` and to log DEBUG (and above) messages from class `com.foo.bar.Clazz` to console:
```json
{
    "loggers": [{
            "packageName": "com.foo.bar",
            "level": "WARN",
            "appender": "file:logs/component.log"
        }, {
            "className": "com.foo.bar.Clazz",
            "level": "DEBUG",
            "appender": "console"
        }
    ]
}
```
You can change configuration file path:
```java
LoggerConfigurationPathHolder.getInstance().setConfigPath("classpath:/log-conf.json");
```
LJL can replace property placeholders in configuration path: `${catalina.base}/logs/app.log`.

### Blocking/Non-blocking invocation settings
By default, LJL using async logging model. If you want to use blocking model, you have to set "async" property to "false":
```json
{
    "async": "false",
    "loggers": [...]
}
```
For async invocations, LJL uses thread pool. You can configure thread pool size:
```json
{
    "async": "true",
    "poolSize": "8",
    "loggers": [...]
}
```

### Log messages format settings
To define the message format, LJL has the predefined pattern variables: `$level$`, `$thread$`, `$class$`, `$date$`, `$msg$`. Date format also can be configured: `$date:patter$`, where `pattern` is `java.util.SimpleDateFormat` constructor parameter.
```json
{
    "loggers": [{
            "packageName": "com.foo.bar",
            "level": "WARN",
            "appender": "file:logs/component.log",
            "format": "[$level$] $thread$ $class$ $date:yyyy-MM-dd HH:mm:ss$ $msg$"
        }
    ]
}
```

### Max backup files settings
You can configure max backup files count.
```json
{
    "loggers": [{
            "packageName": "com.foo.bar",
            "level": "WARN",
            "appender": "file:logs/component.log",
            "maxFileSize": "2 MB",
            "maxFilesCount": "3"
        }
    ]
}
```
Max file size can be configured in kB too.

Servlet container integration
-----------------------------
If you are working in servlet container, configuration file path can be configured like this:
```xml
<context-param>
    <param-name>loggerConfigPath</param-name>
    <param-value>classpath:/logger/ljl.json</param-value>
</context-param>

<listener>
    <listener-class>com.github.dreambrother.ljl.servlet.LjlContextListener</listener-class>
</listener>
```
For this feature your project must have `lightweight-java-logger-servlet-extension` in the classpath.

Spring integration
------------------
Configuration file path can be configured using Spring also. Example:
```xml
<bean id="ljlConfig" class="com.github.dreambrother.ljl.LoggerConfigurationPathHolder" factory-method="getInstance">
    <property name="configPath" value="..."/>
</bean>
```

License
---------
[Apache License, Version 2.0](http://www.apache.org/licenses/LICENSE-2.0)
