/**
 * Copyright 2012-2013 Nikita Shmakov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.dreambrother.ljl;

import com.github.dreambrother.ljl.impl.LoggerImpl;
import com.github.dreambrother.ljl.impl.configuration.LoggerConfigurationOperations;
import com.github.dreambrother.ljl.impl.configuration.LoggerConfigurationOperationsImpl;
import com.github.dreambrother.ljl.impl.configuration.reader.ConfigurationReader;
import com.github.dreambrother.ljl.impl.configuration.reader.ConfigurationReaderImpl;
import com.github.dreambrother.ljl.impl.configuration.reader.FileListener;
import com.github.dreambrother.ljl.impl.defaults.DefaultLoggerConfigurationOperations;
import com.github.dreambrother.ljl.impl.utils.FileUtils;
import java.io.File;

/**
 * Logger factory.
 *
 * @author nik
 */
public final class LoggerFactory {

    private static final String DEFAULT_CONFIG_PATH = "ljl-properties.json";
    private static final LoggerFactory INSTANCE = new LoggerFactory();
    private ConfigurationReader reader = new ConfigurationReaderImpl();
    private volatile LoggerConfigurationOperations loggerConfiguration;
    private FileListener fileListener;

    private LoggerFactory() {
    }

    private static LoggerFactory getInstance() {
        if (!INSTANCE.isConfigured()) {
            INSTANCE.configure();
        }
        return INSTANCE;
    }

    /**
     * Get logger for the specified class.
     *
     * @param clazz
     * @return configured logger
     */
    public static Logger getLogger(Class<?> clazz) {
        return new LoggerImpl(clazz, getInstance().getConfigurationOperations());
    }

    private LoggerConfigurationOperations getConfigurationOperations() {
        if (loggerConfiguration == null) {
            throw new IllegalStateException("LoggerConfigurationOperations is not initialized!");
        }
        return loggerConfiguration;
    }

    private boolean isConfigured() {
        return loggerConfiguration != null;
    }

    private synchronized void configure() {
        if (!isConfigured()) {
            final String configuredPath = LoggerConfigurationPathHolder.getInstance().getConfigPath();
            final String configPath = configuredPath != null ? configuredPath : DEFAULT_CONFIG_PATH;
            final File configFile = FileUtils.asFile(configPath);
            if (configuredPath != null && !configFile.exists()) {
                throw new IllegalStateException(String.format("Configuration file {%s} not found", configuredPath));
            }
            if (configFile.exists()) {
                loggerConfiguration = new LoggerConfigurationOperationsImpl(reader.read(configPath));
                fileListener = new FileListener(configFile, new Runnable() {
                    @Override
                    public void run() {
                        loggerConfiguration.setState(reader.read(configPath));
                    }
                });
                // start listening configuration file
                fileListener.startListening();
            } else { // load defaults
                loggerConfiguration = new DefaultLoggerConfigurationOperations();
            }
        }
    }
}
