/**
 * Copyright 2012-2013 Nikita Shmakov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.dreambrother.ljl.impl.appender;

import com.github.dreambrother.ljl.impl.formatter.LogFormatter;
import com.github.dreambrother.ljl.impl.utils.FileUtils;
import com.github.dreambrother.ljl.impl.utils.SystemPropertyUtils;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.concurrent.Executor;

/**
 * @author nik
 */
public class FileAppender extends AbstractAppender {

    private static final Object LOCK = new Object();
    
    private final long maxFileSize;
    private final int maxFilesCount;
    private final File logFile;

    /**
     * @param executor
     * @param formatter
     * @param filePath
     * @param maxFileSize max file size in bytes, 0 - unlimited file
     * @param maxFilesCount max log files count, 0 - unlimited
     */
    public FileAppender(Executor executor, LogFormatter formatter, String filePath, 
            long maxFileSize, int maxFilesCount) {
        super(executor, formatter);
        this.logFile = new File(SystemPropertyUtils.replaceProperties(filePath));
        FileUtils.createParentDirectories(logFile);
        this.maxFileSize = maxFileSize;
        this.maxFilesCount = maxFilesCount;
    }
    
    @Override
    public void append(String message) {
        if (maxFileSize != 0 && logFile.length() >= maxFileSize) { // if max size was exceeded - remove file
            synchronized (LOCK) {
                if (logFile.length() >= maxFileSize) { // if another thread is not already do it
                    if (maxFilesCount != 0) {
                        int currentLastFile = 0;
                        for (int i = 1; i <= maxFilesCount; i++) {
                            if (new File(logFile.getAbsolutePath() + "." + i).exists()) {
                                currentLastFile = i;
                            }
                        }
                        if (currentLastFile != 0) {
                            if (currentLastFile + 1 <= maxFilesCount) { // if can create next file
                                incrementExtensions(currentLastFile + 1);
                            } else { // if not - rename penult log file to last log file
                                new File(logFile.getAbsolutePath() + "." + currentLastFile).delete();
                                incrementExtensions(currentLastFile);
                            }
                        } else { // just add *.1 extension to log file
                            addExtenstion(logFile, 1);
                        }
                    }
                    logFile.delete();
                }
            }
        }
        try (FileWriter fw = new FileWriter(logFile, true)) {
            fw.write(message + "\n");
        } catch (IOException ex) {
            throw new IllegalStateException("Error while writing in " + logFile, ex);
        }
    }

    private void incrementExtensions(int maxExtensionNumber) {
        for (int i = maxExtensionNumber - 1; i > 0; i--) { // rename all log files
            addExtenstion(new File(logFile.getAbsolutePath() + "." + i), i + 1);
        }
        addExtenstion(logFile, 1);
    }
    
    private void addExtenstion(File file, int value) {
        file.renameTo(new File(file.getAbsolutePath().replaceFirst("\\.\\d+$", "") + "." + value));
    }

    public long getMaxFileSize() {
        return maxFileSize;
    }

    public String getFilePath() {
        return logFile.getAbsolutePath();
    }

    public int getMaxFilesCount() {
        return maxFilesCount;
    }
}
