/**
 * Copyright 2012-2013 Nikita Shmakov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.dreambrother.ljl.impl.configuration;

import com.github.dreambrother.ljl.impl.CurrentThreadExecutor;
import com.github.dreambrother.ljl.impl.LoggingLevel;
import com.github.dreambrother.ljl.impl.appender.ConsoleAppender;
import com.github.dreambrother.ljl.impl.appender.FileAppender;
import com.github.dreambrother.ljl.impl.appender.LogAppender;
import com.github.dreambrother.ljl.impl.configuration.reader.LoggerConfigurationState;
import com.github.dreambrother.ljl.impl.configuration.reader.LoggerConfigurationUnit;
import com.github.dreambrother.ljl.impl.formatter.LogFormatter;
import com.github.dreambrother.ljl.impl.formatter.LogFormatterImpl;
import com.github.dreambrother.ljl.impl.formatter.SimpleLogFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;

/**
 * @author nik
 */
public class LoggerConfigurationOperationsImpl implements LoggerConfigurationOperations {
    
    // thread pool settings are not implement yet
    private static final LogFormatter SIMPLE_LOG_FORMATTER = new SimpleLogFormatter();
    
    private static final String CONSOLE_APPENDER_VALUE = "console";
    private static final String FILE_APPENDER_PREFIX = "file:";
    
    private Executor executor;
    private LoggerConfigurationState state;

    public LoggerConfigurationOperationsImpl(LoggerConfigurationState state) {
        this.executor = getExecutor(state.isAsync(), state.getPoolSize());
        this.state = state;
    }

    @Override
    public synchronized void setState(LoggerConfigurationState state) {
        this.executor = getExecutor(state.isAsync(), state.getPoolSize());
        this.state = state;
    }

    @Override
    public List<LogAppender> getAppenders(Class<?> forClass, LoggingLevel level) {
        List<LogAppender> result = new ArrayList<>();
        List<LoggerConfigurationUnit> loggers = state.getLoggers();
        for (LoggerConfigurationUnit conf : loggers) {
            boolean isClassesEquals = forClass.getName().equals(conf.getClassName());
            boolean isPackagesEquals = conf.getPackageName() != null 
                    && forClass.getName().startsWith(conf.getPackageName());
            boolean isLoggingAllowed = (level == conf.getLevel() || conf.getLevel().getCompatibleLevels().contains(level));
            if ((isClassesEquals || isPackagesEquals) && isLoggingAllowed) {
                result.add(getAppender(conf));
            }
        }
        return result;
    }
    
    private LogAppender getAppender(LoggerConfigurationUnit conf) {
        final String appenderConf = conf.getAppender();
        if (appenderConf.equals(CONSOLE_APPENDER_VALUE)) {
            return new ConsoleAppender(executor, getFormatter(conf));
        } else if (appenderConf.startsWith(FILE_APPENDER_PREFIX)) {
            if (conf.getMaxFilesCount() < 0) {
                throw new IllegalStateException("Invalid max files count settings " + conf.getMaxFilesCount());
            }
            final String filePath = appenderConf.replace(FILE_APPENDER_PREFIX, "");
            return new FileAppender(executor, getFormatter(conf), filePath, 
                    toBytes(conf.getMaxFileSize()), conf.getMaxFilesCount());
        } else {
            throw new IllegalArgumentException("Unknown appender: " + appenderConf);
        }
    }
    
    private LogFormatter getFormatter(LoggerConfigurationUnit conf) {
        if (conf.getFormat() != null) {
            return new LogFormatterImpl(conf.getFormat());
        } else {
            return SIMPLE_LOG_FORMATTER;
        }
    }
    
    private long toBytes(String maxFileSize) {
        if (maxFileSize != null) {
            String[] splittedSize = maxFileSize.split(" ");
            if (splittedSize.length != 2) {
                throw new IllegalStateException("Invalid max file size settings " + maxFileSize);
            }
            final int size = Integer.valueOf(splittedSize[0]);
            if (size < 0) {
                throw new IllegalStateException("Invalid max file size value " + size);
            }
            switch (splittedSize[1]) {
                case "MB":
                    return size * 1024 * 1024;
                case "kB":
                    return size * 1024;
                default:
                    throw new IllegalStateException("Invalid max file size settings format " + maxFileSize);
            }
        } else {
            return 0L;
        }
    }
    
    private Executor getExecutor(boolean isAsync, int poolSize) {
        if (!isAsync) {
            return new CurrentThreadExecutor();
        }
        if (poolSize == 0) {
            return Executors.newCachedThreadPool();
        } else if (poolSize > 0) {
            return Executors.newFixedThreadPool(poolSize);
        } else {
            throw new IllegalArgumentException("Invalid pool size settings " + poolSize);
        }
    }
}
