/**
 * Copyright 2012-2013 Nikita Shmakov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.dreambrother.ljl.impl.configuration.reader;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.StandardWatchEventKinds;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;

/**
 *
 * @author nik
 */
public class FileListener {

    private final File file;
    private final Runnable callback;
    private Thread listenerThread;

    public FileListener(File file, Runnable eventCallback) {
        this.file = file;
        this.callback = eventCallback;
    }

    public synchronized void startListening() {
        if (listenerThread != null) {
            throw new IllegalStateException("Listener was already activated");
        }
        listenerThread = new Thread(new Runnable() {
            @Override
            public void run() {
                Path path = file.getAbsoluteFile().getParentFile().toPath();
                try (WatchService ws = path.getFileSystem().newWatchService()) {
                    path.register(ws, StandardWatchEventKinds.ENTRY_MODIFY);
                    while (true) {
                        WatchKey take = ws.take();
                        for (WatchEvent<?> event : take.pollEvents()) {
                            callback.run();
                        }
                        if (!take.reset()) {
                            break;
                        }
                    }
                } catch (InterruptedException ex) {
                    Thread.currentThread().interrupt();
                } catch (IOException ex) {
                    // log stacktrace into the System.err and kill the thread
                    throw new IllegalStateException(ex);
                }
            }
        });
        listenerThread.start();
    }
    
    public synchronized void stopListening() {
        if (listenerThread == null) {
            throw new IllegalStateException("Listener was not activated");
        }
        listenerThread.interrupt();
    }
}
