/**
 * Copyright 2012-2013 Nikita Shmakov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.dreambrother.ljl.impl.formatter;

import com.github.dreambrother.ljl.impl.LoggingLevel;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Uses {@link SimpleDateFormat} for date formatting.
 * @author nik
 */
public class LogFormatterImpl extends AbstractLogFormatter {
    
    public static final String CONTAINS_THREAD_VAR_REGEXP = ".*\\$thread\\$.*";
    public static final String CONTAINS_CLASS_VAR_REGEXP = ".*\\$class\\$.*";
    public static final String CONTAINS_LEVEL_VAR_REGEXP = ".*\\$level\\$.*";
    public static final String CONTAINS_MESSAGE_VAR_REGEXP = ".*\\$msg\\$.*";
    public static final String CONTAINS_DATE_VAR_REGEXP = ".*\\$date\\$.*";
    public static final String CONTAINS_DATE_VAR_WITH_PARAM_REGEXP = ".*\\$date:.*\\$.*";
    
    public static final String THREAD_VAR_REGEXP = "\\$thread\\$";
    public static final String CLASS_VAR_REGEXP = "\\$class\\$";
    public static final String LEVEL_VAR_REGEXP = "\\$level\\$";
    public static final String MESSAGE_VAR_REGEXP = "\\$msg\\$";
    public static final String DATE_VAR_REGEXP = "\\$date\\$";
    public static final String DATE_VAR_WITH_PARAM_REGEXP = "(\\$date:.*?\\$)";
    
    private final String pattern;
    private final String datePattern;
    private final boolean threadVarPresents;
    private final boolean classVarPresents;
    private final boolean levelVarPresents;
    private final boolean messageVarPresents;
    private final boolean dateVarPresents;

    public LogFormatterImpl(String pattern) {
        this.pattern = pattern;
        threadVarPresents = pattern.matches(CONTAINS_THREAD_VAR_REGEXP);
        classVarPresents = pattern.matches(CONTAINS_CLASS_VAR_REGEXP);
        levelVarPresents = pattern.matches(CONTAINS_LEVEL_VAR_REGEXP);
        messageVarPresents = pattern.matches(CONTAINS_MESSAGE_VAR_REGEXP);
        dateVarPresents = pattern.matches(CONTAINS_DATE_VAR_REGEXP);
        if (pattern.matches(CONTAINS_DATE_VAR_WITH_PARAM_REGEXP)) {
            Pattern regexpPattern = Pattern.compile(DATE_VAR_WITH_PARAM_REGEXP);
            Matcher matcher = regexpPattern.matcher(pattern);
            if (matcher.groupCount() != 1) {
                throw new IllegalStateException("Invalid $date$ pattern count " + matcher.groupCount());
            }
            if (!matcher.find()) {
                throw new IllegalStateException("Invalid pattern " + pattern);
            }
            String patternSubstring = matcher.group();
            datePattern = patternSubstring.substring("$date:".length(), patternSubstring.length() - 1);
        } else {
            datePattern = null;
        }
    }

    @Override
    protected String performConcreteFormat(LoggingLevel level, Class<?> clazz, Date date, String message) {
        String result = pattern;
        if (threadVarPresents) {
            result = result.replaceAll(THREAD_VAR_REGEXP, Thread.currentThread().getName());
        }
        if (classVarPresents) {
            result = result.replaceAll(CLASS_VAR_REGEXP, clazz.getName());
        }
        if (levelVarPresents) {
            result = result.replaceAll(LEVEL_VAR_REGEXP, level.name());
        }
        if (messageVarPresents) {
            result = result.replaceAll(MESSAGE_VAR_REGEXP, message);
        }
        if (datePattern != null) {
            SimpleDateFormat df = new SimpleDateFormat(datePattern);
            result = result.replaceAll(DATE_VAR_WITH_PARAM_REGEXP, df.format(new Date()));
        } else if (dateVarPresents) {
            result = result.replaceAll(DATE_VAR_REGEXP, new Date().toString());
        }
        return result;
    }
}
