/**
 * Copyright 2012-2013 Nikita Shmakov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.dreambrother.ljl.impl.appender;

import com.github.dreambrother.ljl.impl.LoggingLevel;
import com.github.dreambrother.ljl.impl.formatter.LogFormatter;
import com.github.dreambrother.ljl.impl.CurrentThreadExecutor;
import com.github.dreambrother.ljl.impl.util.TestClass;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Date;
import java.util.concurrent.Executor;
import org.easymock.EasyMock;
import org.junit.After;
import org.junit.Test;

import static org.easymock.EasyMock.*;
import static org.junit.Assert.*;

/**
 *
 * @author nik
 */
public class FileAppenderIntTest {

    private final File testFile = new File("test-file");
    private final File testFile1 = new File("test-file.1");
    private final File testFile2 = new File("test-file.2");
    private final LogFormatter formatterMock = EasyMock.createMock(LogFormatter.class);
    private final Executor executor = new CurrentThreadExecutor();
    
    @After
    public void removeFile() {
        if (testFile.exists()) {
            testFile.delete();
        }
        if (testFile1.exists()) {
            testFile1.delete();
        }
        if (testFile2.exists()) {
            testFile2.delete();
        }
    }
    
    @Test
    public void testMaxFileSizeExceeding() {
        final String message = "log message";
        reset(formatterMock);
        expect(formatterMock.format(eq(LoggingLevel.ERROR), eq(TestClass.class), anyObject(Date.class), eq(message)))
                .andReturn(message).anyTimes();
        replay(formatterMock);
        
        final long maxFileSize = 2 * 1024; // 2 kB
        final LogAppender appender = new FileAppender(executor, formatterMock, testFile.getPath(), 
                maxFileSize, 0);
        for (long bytesWritten = 0; bytesWritten < maxFileSize; bytesWritten += 
                (message.getBytes().length + "\n".getBytes().length)) {
            appender.append(LoggingLevel.ERROR, TestClass.class, message);
        }
        assertTrue(testFile.length() > maxFileSize);
        appender.append(LoggingLevel.ERROR, TestClass.class, message); // log file will be re-created
        assertTrue(testFile.length() < maxFileSize);
        
        verify(formatterMock);
    }
    
    @Test
    public void testMaxFilesCountFeature() {
        final String firstMessage = new String(new byte[]{1, 2, 3, 4, 5});
        final String secondMessage = new String(new byte[]{2, 3, 4, 5, 6});
        final String thirdMessage = new String(new byte[]{4, 5, 6, 7, 8});
        final String fourthMessage = new String(new byte[]{5, 6, 7, 8, 9});
        final long maxFileSize = 5 + "\n".getBytes().length;
        reset(formatterMock);
        expect(formatterMock.format(eq(LoggingLevel.ERROR), eq(TestClass.class), anyObject(Date.class), 
                    eq(firstMessage))).andReturn(firstMessage);
        expect(formatterMock.format(eq(LoggingLevel.ERROR), eq(TestClass.class), anyObject(Date.class), 
                    eq(secondMessage))).andReturn(secondMessage);
        expect(formatterMock.format(eq(LoggingLevel.ERROR), eq(TestClass.class), anyObject(Date.class), 
                    eq(thirdMessage))).andReturn(thirdMessage);
        expect(formatterMock.format(eq(LoggingLevel.ERROR), eq(TestClass.class), anyObject(Date.class), 
                    eq(fourthMessage))).andReturn(fourthMessage);
        replay(formatterMock);
        
        final int maxFiles = 2;
        
        final LogAppender appender = new FileAppender(executor, formatterMock, testFile.getPath(), 
                maxFileSize, maxFiles);
        appender.append(LoggingLevel.ERROR, TestClass.class, firstMessage);
        assertEquals(maxFileSize, testFile.length());
        assertEquals(firstMessage, readFile(testFile));
        
        appender.append(LoggingLevel.ERROR, TestClass.class, secondMessage);
        // log file with *.1 extension should be created
        assertTrue(testFile1.exists());
        assertEquals(maxFileSize, testFile1.length());
        // contents of the main log file must be equals to new log message
        assertEquals(secondMessage, readFile(testFile));
        assertEquals(firstMessage, readFile(testFile1));
        
        appender.append(LoggingLevel.ERROR, TestClass.class, thirdMessage);
        // log file with *.2 extension should be created
        assertTrue(testFile2.exists());
        assertEquals(maxFileSize, testFile2.length());
        // contents of the main log file must be equals to new log message
        assertEquals(thirdMessage, readFile(testFile));
        assertEquals(secondMessage, readFile(testFile1));
        assertEquals(firstMessage, readFile(testFile2));
        
        appender.append(LoggingLevel.ERROR, TestClass.class, fourthMessage);
        assertFalse(new File(testFile.getAbsolutePath() + ".4").exists()); // new file should not have been created
        // contents of the main log file must be equals to new log message
        assertEquals(fourthMessage, readFile(testFile));
        assertEquals(thirdMessage, readFile(testFile1));
        assertEquals(secondMessage, readFile(testFile2));
        
        verify(formatterMock);
    }
    
    private String readFile(File file) {
        try {
            try (final BufferedReader reader = new BufferedReader(new FileReader(file))) {
                String line;
                String result = "";
                while ((line = reader.readLine()) != null) {
                    result += line;
                }
                return result;
            } catch (FileNotFoundException ex) {
                throw new AssertionError("Something terrible was happened", ex);
            }
        } catch (IOException ex) {
            throw new AssertionError("Something terrible was happened", ex);
        }
    }
}
