/**
 * Copyright 2012-2013 Nikita Shmakov
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.github.dreambrother.ljl.impl.configuration;

import com.github.dreambrother.ljl.impl.CurrentThreadExecutor;
import com.github.dreambrother.ljl.impl.LoggingLevel;
import com.github.dreambrother.ljl.impl.appender.AbstractAppender;
import com.github.dreambrother.ljl.impl.appender.ConsoleAppender;
import com.github.dreambrother.ljl.impl.appender.FileAppender;
import com.github.dreambrother.ljl.impl.appender.LogAppender;
import com.github.dreambrother.ljl.impl.configuration.reader.LoggerConfigurationState;
import com.github.dreambrother.ljl.impl.configuration.reader.LoggerConfigurationUnit;
import com.github.dreambrother.ljl.impl.formatter.LogFormatter;
import com.github.dreambrother.ljl.impl.formatter.LogFormatterImpl;
import com.github.dreambrother.ljl.impl.util.TestClass;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import org.junit.Test;

import static org.junit.Assert.*;

/**
 * @author nik
 */
public class LoggerConfigurationOperationsImplTest {
    
    @Test
    public void testGetAppenders() {
        final List<LoggerConfigurationUnit> configurations = new ArrayList<>();
        configurations.add(new LoggerConfigurationUnit("com.github.dreambrother", null, 
                "console", LoggingLevel.DEBUG, null, null, 0));
        configurations.add(new LoggerConfigurationUnit("com.github.dreambrother", null, 
                "file:/test/errors-log", LoggingLevel.WARN, null, null, 0));
        configurations.add(new LoggerConfigurationUnit(null, "com.github.dreambrother.ljl.impl.util.TestClass", 
                "file:/test/TestClass-log", LoggingLevel.INFO, null, null, 0));
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0, configurations);
        final LoggerConfigurationOperations configurationOperations = 
                new LoggerConfigurationOperationsImpl(configurationState);
        
        List<LogAppender> appenders = configurationOperations.getAppenders(TestClass.class, LoggingLevel.DEBUG);
        assertEquals(1, appenders.size());
        assertTrue(appenders.get(0) instanceof ConsoleAppender);
        
        appenders = configurationOperations.getAppenders(TestClass.class, LoggingLevel.INFO);
        assertEquals(2, appenders.size());
        assertTrue(appenders.get(0) instanceof ConsoleAppender);
        assertTrue(appenders.get(1) instanceof FileAppender);
        assertEquals("/test/TestClass-log", ((FileAppender) appenders.get(1)).getFilePath());
        
        appenders = configurationOperations.getAppenders(TestClass.class, LoggingLevel.ERROR);
        assertEquals(3, appenders.size());
        assertTrue(appenders.get(0) instanceof ConsoleAppender);
        assertTrue(appenders.get(1) instanceof FileAppender);
        assertTrue(appenders.get(2) instanceof FileAppender);
        assertEquals("/test/errors-log", ((FileAppender) appenders.get(1)).getFilePath());
        assertEquals("/test/TestClass-log", ((FileAppender) appenders.get(2)).getFilePath());
        
        appenders = configurationOperations.getAppenders(String.class, LoggingLevel.ERROR);
        assertTrue(appenders.isEmpty());
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void testInvalidAppender() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0, 
                Collections.singletonList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "jdbc" /* invalid appender */, LoggingLevel.DEBUG, null, null, 0)));
        final LoggerConfigurationOperations configurationOperations = new LoggerConfigurationOperationsImpl(configurationState);
        configurationOperations.getAppenders(TestClass.class, LoggingLevel.ERROR);
    }
    
    @Test
    public void testGetFormatter() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0, 
                Collections.singletonList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "console", LoggingLevel.DEBUG, "${msg}", null, 0)));
        final LoggerConfigurationOperations configurationOperations = new LoggerConfigurationOperationsImpl(configurationState);
        LogFormatter formatter = ((AbstractAppender) configurationOperations.getAppenders(
                TestClass.class, LoggingLevel.DEBUG).get(0)).getFormatter();
        assertTrue(formatter instanceof LogFormatterImpl);
    }
    
    @Test
    public void testMaxFileSizeSettings() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0,
                Arrays.asList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "file:/test", LoggingLevel.DEBUG, null, "2 MB", 1),
                              new LoggerConfigurationUnit("java.lang", 
                    null, "file:/test-error", LoggingLevel.ERROR, null, "16 kB", 2)));
        final LoggerConfigurationOperations configurationOperations = new LoggerConfigurationOperationsImpl(configurationState);
        
        List<LogAppender> appenders = configurationOperations.getAppenders(TestClass.class, LoggingLevel.DEBUG);
        assertTrue(appenders.size() == 1);
        assertEquals(2 * 1024 * 1024, ((FileAppender) appenders.get(0)).getMaxFileSize());
        
        appenders = configurationOperations.getAppenders(String.class, LoggingLevel.ERROR);
        assertTrue(appenders.size() == 1);
        assertEquals(16 * 1024, ((FileAppender) appenders.get(0)).getMaxFileSize());
    }
    
    @Test(expected = IllegalStateException.class)
    public void testInvalidFileSizeSettings() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0,
                Collections.singletonList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "file:/test", LoggingLevel.DEBUG, "${msg}", "16 GB" /* Invalid size */, 0)));
        final LoggerConfigurationOperations configurationOperations = new LoggerConfigurationOperationsImpl(configurationState);
        configurationOperations.getAppenders(TestClass.class, LoggingLevel.ERROR);
    }
    
    @Test(expected = IllegalStateException.class)
    public void testIllegalStateExceptionWhenFileSizeIsNegative() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0,
                Collections.singletonList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "file:/test", LoggingLevel.DEBUG, "${msg}", "-1 GB" /* Invalid size */, 0)));
        final LoggerConfigurationOperations configurationOperations = new LoggerConfigurationOperationsImpl(configurationState);
        configurationOperations.getAppenders(TestClass.class, LoggingLevel.ERROR);
    }
    
    @Test(expected = IllegalStateException.class)
    public void testInvalidMaxFilesCount() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0,
                Collections.singletonList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "file:/test", LoggingLevel.DEBUG, "${msg}", "1 GB", -1 /* Invalid count */)));
        final LoggerConfigurationOperations configurationOperations = new LoggerConfigurationOperationsImpl(configurationState);
        configurationOperations.getAppenders(TestClass.class, LoggingLevel.ERROR);
    }
    
    @Test(expected = IllegalArgumentException.class)
    public void testInvalidPoolSizeSettings() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(true, -1,
                Collections.singletonList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "file:/test", LoggingLevel.DEBUG, "${msg}", "1 GB", -1 /* Invalid count */)));
        new LoggerConfigurationOperationsImpl(configurationState);
    }
    
    @Test
    public void shouldUseCurrentThreadExecutorWhenAsyncIsFalse() {
        final LoggerConfigurationState configurationState = new LoggerConfigurationState(false, 0, 
                Collections.singletonList(new LoggerConfigurationUnit("com.github.dreambrother", 
                    null, "console", LoggingLevel.DEBUG, "${msg}", null, 0)));
        final LoggerConfigurationOperations configurationOperations = new LoggerConfigurationOperationsImpl(configurationState);
        List<LogAppender> appenders = configurationOperations.getAppenders(TestClass.class, LoggingLevel.ERROR);
        assertTrue(((AbstractAppender) appenders.get(0)).getExecutor() instanceof CurrentThreadExecutor);
    }
}
